﻿/* USER CODE BEGIN Header */
/**
  ******************************************************************************
  * @file           : main.c
  * @brief          : Main program body
  ******************************************************************************
  * @attention
  *
  * Copyright (c) 2025 STMicroelectronics.
  * All rights reserved.
  *
  * This software is licensed under terms that can be found in the LICENSE file
  * in the root directory of this software component.
  * If no LICENSE file comes with this software, it is provided AS-IS.
  *
  ******************************************************************************
  */
/* USER CODE END Header */
/* Includes ------------------------------------------------------------------*/
#include "main.h"
#include "adc.h"
#include "can.h"
#include "i2c.h"
#include "rtc.h"
#include "spi.h"
#include "tim.h"
#include "usart.h"
#include "gpio.h"

/* Private includes ----------------------------------------------------------*/
/* USER CODE BEGIN Includes */
#include "dallas_tools.h"

#include "def.h"
#include <stdio.h>
#include "modbus.h"
#include "eeprom_emul.h"
#include "stdio.h"
#include "flash_ring.h"
#include "string.h"




/* USER CODE END Includes */

/* Private typedef -----------------------------------------------------------*/
/* USER CODE BEGIN PTD */

/* USER CODE END PTD */

/* Private define ------------------------------------------------------------*/
/* USER CODE BEGIN PD */
uint16_t iter, cnt = 5;
uint8_t init_retries = 5;
uint8_t ralay_5v_on_var = 0;
/* USER CODE END PD */

/* Private macro -------------------------------------------------------------*/
/* USER CODE BEGIN PM */
float temperature;
uint8_t roms[MAX_DEVICES][8];
Flags_TypeDef flag;
//extern uint8_t devices_found ;
uint8_t _debug_init = 0;
TEMP_TypeDef temp_sense[30];
float set_temp_old[30];
char rx_buffer[64];
uint8_t rx_index = 0;
char command_ready = 0;
uint8_t uart_byte = 0;
uint8_t first_in = 1;
DALLAS_SensorHandleTypeDef sens[30];
int init_sens = 0;
FlashRecord_t* record;
uint8_t flash_buff[RECORD_SIZE - 4];

RS_HandleTypeDef hmodbus_master;
RS_MsgTypeDef MODBUS_MASTER_MSG;
static uint8_t hmodbus_master_buffer[MSG_SIZE_MAX];
static uint8_t rtc_modbus_hold_initialized = 0U;

/* USER CODE END PM */

/* Private variables ---------------------------------------------------------*/

/* USER CODE BEGIN PV */
int last_page_addr = LAST_PAGE_ADDR;

/* USER CODE END PV */

/* Private function prototypes -----------------------------------------------*/
void SystemClock_Config(void);
/* USER CODE BEGIN PFP */
static void RTC_ModbusService(MB_DataStructureTypeDef* mb_data);
static void RTC_ReadCalendarFromRegs(const MB_RtcCalendarRegsTypeDef* regs, RTC_CalendarTypeDef* calendar);
static void RTC_WriteCalendarToRegs(MB_RtcCalendarRegsTypeDef* regs, const RTC_CalendarTypeDef* calendar, uint16_t status);

/* USER CODE END PFP */

/* Private user code ---------------------------------------------------------*/
/* USER CODE BEGIN 0 */
HAL_StatusTypeDef ModbusMaster_Request(RS_MsgTypeDef *request,
                                        void (*callback)(RS_HandleTypeDef *, RS_MsgTypeDef *))
{
    return MODBUS_MasterRequest(&hmodbus_master, request, callback);
}

/* USER CODE END 0 */

/**
  * @brief  The application entry point.
  * @retval int
  */
int main(void)
{

  /* USER CODE BEGIN 1 */

  /* USER CODE END 1 */

  /* MCU Configuration--------------------------------------------------------*/

  /* Reset of all peripherals, Initializes the Flash interface and the Systick. */
  HAL_Init();

  /* USER CODE BEGIN Init */

  /* USER CODE END Init */

  /* Configure the system clock */
  SystemClock_Config();

  /* USER CODE BEGIN SysInit */


  /* USER CODE END SysInit */

  /* Initialize all configured peripherals */
  MX_GPIO_Init();
  MX_TIM1_Init();
  MX_USART1_UART_Init();
  MX_USART2_UART_Init();
  MX_TIM2_Init();
  MX_TIM4_Init();
  MX_ADC1_Init();
  MX_CAN_Init();
  MX_I2C1_Init();
  MX_RTC_Init();
  MX_SPI1_Init();
  /* USER CODE BEGIN 2 */



    led_blink(GPIOC, 13, rest_iter, reset_blink_delay);
    MODBUS_FirstInit(&hmodbus1, &mb_huart, &mb_htim);
    MODBUS_Config(&hmodbus1, MODBUS_DEVICE_ID, MODBUS_TIMEOUT, MODBUS_MODE_SLAVE);
    // Запуск приема Modbus
    MODBUS_SlaveStart(&hmodbus1, NULL);
    if (MODBUS_FirstInit(&hmodbus_master, &huart2, &htim4) != HAL_OK)
    {
        Error_Handler();
    }
    hmodbus_master.pBufferPtr = hmodbus_master_buffer;
    if (MODBUS_Config(&hmodbus_master, 0, MODBUS_TIMEOUT, MODBUS_MODE_MASTER) != HAL_OK)
    {
        Error_Handler();
    }
    uint8_t uart_byte = 0;
    Dallas_BusFirstInit(&htim1);
    // ������������� �� ������� ��������� ��������
    // ������������� �� ������� (����������� ������ ���������� �������)
    reinit_t_sens();
    init_setpoint_all_T_sense(temp_sense, hdallas.onewire->RomCnt);
		
    MB_DATA.InRegs.num_Tsens = hdallas.onewire->RomCnt;
    RTC_ModbusService(&MB_DATA);
   // BufferState_t buffer_state = buffer_init();
for(int i=0;i<RECORD_SIZE;i++)
{
	flash_buff[i]=i;
	
}
  /* USER CODE END 2 */

  /* Infinite loop */
  /* USER CODE BEGIN WHILE */
    while (1)
    {


//        if (MB_DATA.Coils.relay_struct[0].state_val_bit.Temp11_relay_isOn)
//        {
//            MB_DATA.Coils.relay_struct[0].state_val_bit.Temp11_relay_isOn = 0;
//            BufferState_t state = buffer_init();
//            uint32_t idx = (state.write_index ) % RECORDS_PER_PAGE;
//            FlashRecord_t* record = buffer_read_record(idx);

//        }
//        if (MB_DATA.Coils.relay_struct[0].state_val_bit.Temp10_relay_isOn)
//        {
//            MB_DATA.Coils.relay_struct[0].state_val_bit.Temp10_relay_isOn = 0;
//            FlashRecord_t new_record;
//            new_record.timestamp = HAL_GetTick();
//            memcpy(new_record.data, flash_buff, sizeof(new_record.data));

//            HAL_StatusTypeDef status = buffer_write_record(&new_record, &buffer_state);

//            if (status == HAL_OK)
//            {
//                //                          printf("Record written successfully\n");
//                GPIOC->ODR |= 1 << 13;
//            }

//        }
        temp_sense[0].t_close = 1;
        Field_modbus(&MB_DATA, &flag);
        Check_Tconnect(&MB_DATA, &flag, &hdallas, 0);
        value_control();
        init_setpoint_all_T_sense(temp_sense, hdallas.onewire->RomCnt);
        // handle_valves(temp_sense[]);
        Dallas_StartConvertTAll(&hdallas, DALLAS_WAIT_BUS, 0);
        for(int i = 0; i < hdallas.onewire->RomCnt; i++)
        {
            if(sens[i].isLost)
            {
                sens[i].lost_cnt ++;
            }
            Dallas_ReadTemperature(&sens[i]);
            MB_DATA.InRegs.sens_Temp[i] = sens[i].temperature * 10;
/////////////////////////заменить на define ralay_5v_on_var GPIOA->ODR|=1<<10;
            ralay_5v_on_var = MB_DATA.Coils.coils[1].state_val_bit.state_val_05;
            if (ralay_5v_on_var)
            {
                GPIOA->ODR |= 1 << 10;
            }
            else
            {
                GPIOA->ODR &= ~(1 << 10);
            }


        }




    /* USER CODE END WHILE */

    /* USER CODE BEGIN 3 */
        //iwdg_refresh();

        //HAL_Delay(200);
    }
  /* USER CODE END 3 */
}

/**
  * @brief System Clock Configuration
  * @retval None
  */
void SystemClock_Config(void)
{
  RCC_OscInitTypeDef RCC_OscInitStruct = {0};
  RCC_ClkInitTypeDef RCC_ClkInitStruct = {0};
  RCC_PeriphCLKInitTypeDef PeriphClkInit = {0};

  /** Initializes the RCC Oscillators according to the specified parameters
  * in the RCC_OscInitTypeDef structure.
  */
  RCC_OscInitStruct.OscillatorType = RCC_OSCILLATORTYPE_LSI|RCC_OSCILLATORTYPE_HSE;
  RCC_OscInitStruct.HSEState = RCC_HSE_ON;
  RCC_OscInitStruct.HSEPredivValue = RCC_HSE_PREDIV_DIV1;
  RCC_OscInitStruct.HSIState = RCC_HSI_ON;
  RCC_OscInitStruct.LSIState = RCC_LSI_ON;
  RCC_OscInitStruct.PLL.PLLState = RCC_PLL_ON;
  RCC_OscInitStruct.PLL.PLLSource = RCC_PLLSOURCE_HSE;
  RCC_OscInitStruct.PLL.PLLMUL = RCC_PLL_MUL9;
  if (HAL_RCC_OscConfig(&RCC_OscInitStruct) != HAL_OK)
  {
    Error_Handler();
  }

  /** Initializes the CPU, AHB and APB buses clocks
  */
  RCC_ClkInitStruct.ClockType = RCC_CLOCKTYPE_HCLK|RCC_CLOCKTYPE_SYSCLK
                              |RCC_CLOCKTYPE_PCLK1|RCC_CLOCKTYPE_PCLK2;
  RCC_ClkInitStruct.SYSCLKSource = RCC_SYSCLKSOURCE_PLLCLK;
  RCC_ClkInitStruct.AHBCLKDivider = RCC_SYSCLK_DIV1;
  RCC_ClkInitStruct.APB1CLKDivider = RCC_HCLK_DIV2;
  RCC_ClkInitStruct.APB2CLKDivider = RCC_HCLK_DIV1;

  if (HAL_RCC_ClockConfig(&RCC_ClkInitStruct, FLASH_LATENCY_2) != HAL_OK)
  {
    Error_Handler();
  }
  PeriphClkInit.PeriphClockSelection = RCC_PERIPHCLK_RTC|RCC_PERIPHCLK_ADC;
  PeriphClkInit.RTCClockSelection = RCC_RTCCLKSOURCE_LSI;
  PeriphClkInit.AdcClockSelection = RCC_ADCPCLK2_DIV6;
  if (HAL_RCCEx_PeriphCLKConfig(&PeriphClkInit) != HAL_OK)
  {
    Error_Handler();
  }
}

/* USER CODE BEGIN 4 */
static void RTC_ReadCalendarFromRegs(const MB_RtcCalendarRegsTypeDef* regs, RTC_CalendarTypeDef* calendar)
{
    if ((regs == NULL) || (calendar == NULL))
    {
        return;
    }

    calendar->hours = regs->hours;
    calendar->minutes = regs->minutes;
    calendar->seconds = regs->seconds;
    calendar->date = regs->date;
    calendar->month = regs->month;
    calendar->year = regs->year;
    calendar->weekday = regs->weekday;
}

static void RTC_WriteCalendarToRegs(MB_RtcCalendarRegsTypeDef* regs, const RTC_CalendarTypeDef* calendar, uint16_t status)
{
    if ((regs == NULL) || (calendar == NULL))
    {
        return;
    }

    regs->hours = calendar->hours;
    regs->minutes = calendar->minutes;
    regs->seconds = calendar->seconds;
    regs->date = calendar->date;
    regs->month = calendar->month;
    regs->year = calendar->year;
    regs->weekday = calendar->weekday;
    regs->apply = 0U;
    regs->status = status;
}

static void RTC_ModbusService(MB_DataStructureTypeDef* mb_data)
{
    RTC_CalendarTypeDef calendar = {0};
    uint16_t hold_status = MB_RTC_STATUS_IDLE;
    uint8_t copy_current_to_holding = 0U;

    if (mb_data == NULL)
    {
        return;
    }

    hold_status = mb_data->HoldRegs.rtc.status;

    if (mb_data->HoldRegs.rtc.apply == MB_RTC_APPLY_SET)
    {
        RTC_ReadCalendarFromRegs(&mb_data->HoldRegs.rtc, &calendar);
        mb_data->HoldRegs.rtc.apply = 0U;

        if (RTC_CalendarIsValid(&calendar) == 0U)
        {
            hold_status = MB_RTC_STATUS_VALUE_ERROR;
        }
        else if (RTC_CalendarSet(&calendar) != HAL_OK)
        {
            hold_status = MB_RTC_STATUS_HAL_ERROR;
        }
        else
        {
            hold_status = MB_RTC_STATUS_SET_OK;
            copy_current_to_holding = 1U;
        }
    }

    if (RTC_CalendarGet(&calendar) != HAL_OK)
    {
        mb_data->InRegs.rtc.status = MB_RTC_STATUS_HAL_ERROR;
        mb_data->HoldRegs.rtc.status = MB_RTC_STATUS_HAL_ERROR;
        return;
    }

    RTC_WriteCalendarToRegs(&mb_data->InRegs.rtc, &calendar, MB_RTC_STATUS_IDLE);

    if ((rtc_modbus_hold_initialized == 0U) || (copy_current_to_holding != 0U))
    {
        RTC_WriteCalendarToRegs(&mb_data->HoldRegs.rtc, &calendar, hold_status);
        rtc_modbus_hold_initialized = 1U;
    }
    else
    {
        mb_data->HoldRegs.rtc.status = hold_status;
    }
}

void iwdg_refresh(void)
{
    IWDG->KR = 0xAAAA;         // �������� ������
}
void led_blink(GPIO_TypeDef *GPIOx, uint16_t GPIO_Pin, uint8_t iter, uint16_t delay)
{
    for(int i = 0; i < iter; i++)
    {
        GPIOx->ODR ^= (1 << GPIO_Pin);
        HAL_Delay(delay);
    }
}

void  Check_Tconnect(MB_DataStructureTypeDef* MB_DATA,  Flags_TypeDef* flag, DALLAS_HandleTypeDef* hdallas, int a[0])
{
    for(int i = 0; i < hdallas->onewire->RomCnt; i++)
    {
        if(sens[i].isLost)
        {
            //init_sens=1;
        }

    }
    if (init_sens || flag->init_tsens)
    {
        init_sens = 0;
        flag->init_tsens = 0;
        //Dallas_BusFirstInit(&htim1);
        DS18B20_Search(&DS, &OW) ;
        reinit_t_sens();
        MB_DATA->InRegs.num_Tsens = hdallas->onewire->RomCnt;
    }
}

void reinit_t_sens(void)
{
    for ( int i = 0; i < hdallas.onewire->RomCnt; i++)
    {
        // ������������� �� ROM-������
        //sens[i].Init.init_func = &Dallas_SensorInitByROM;
        //  sens[i].Init.InitParam.ROM = rom_address;
        sens[i].Init.InitParam.Ind = i;
        sens[i].Init.init_func = &Dallas_SensorInitByInd;
        sens[i].Init.Resolution = DALLAS_CONFIG_9_BITS;        sens[i].set_temp = 20.;
        sens[i].hyst = 1;
        MB_DATA.HoldRegs.set_Temp[i] = (uint16_t)(sens[i].set_temp * MB_ROOM_TEMP_SCALE);
        MB_DATA.HoldRegs.set_hyst[i] = (uint16_t)(sens[i].hyst * MB_ROOM_TEMP_SCALE);
        MB_DATA.HoldRegs.room_cfg[i].setpoint_x10 = MB_DATA.HoldRegs.set_Temp[i];
        MB_DATA.HoldRegs.room_cfg[i].hysteresis_x10 = MB_DATA.HoldRegs.set_hyst[i];
        MB_DATA.HoldRegs.room_cfg[i].valve_position_pct = 0;
        MB_DATA.HoldRegs.room_cfg[i].valve_angle_max_deg = MB_ROOM_VALVE_ANGLE_MAX_DEFAULT;
        MB_DATA.HoldRegs.room_cfg[i].mode = ROOM_MODE_AUTO;
        MB_DATA.HoldRegs.room_cfg[i].command = ROOM_COMMAND_STOP;
        MB_DATA.HoldRegs.room_cfg[i].location = 1;
        MB_DATA.HoldRegs.room_cfg[i].apply = 0;
        Dallas_AddNewSensors(&hdallas, &sens[i]);

    }
}
FuncStat packStruct(MB_DataStructureTypeDef* MB_DATA, int sizeARR)
{
    for(int i = 0; i < sizeARR; i++)
    {
        for(int sens_num = 0; sens_num < hdallas.onewire->RomCnt; sens_num++)
        {
            switch(sens_num)
            {
            case 0:
                MB_DATA->Coils.status_tSens[i].state_val_bit.Temp1_isConnected = sens[i * 16 + sens_num ].isConnected;
                break;
            case 1:
                MB_DATA->Coils.status_tSens[i].state_val_bit.Temp2_isConnected = sens[i * 16 + sens_num ].isConnected;
                break;
            case 2:
                MB_DATA->Coils.status_tSens[i].state_val_bit.Temp3_isConnected = sens[i * 16 + sens_num ].isConnected;
                break;
            case 3:
                MB_DATA->Coils.status_tSens[i].state_val_bit.Temp4_isConnected = sens[i * 16 + sens_num ].isConnected;
                break;
            case 4:
                MB_DATA->Coils.status_tSens[i].state_val_bit.Temp5_isConnected = sens[i * 16 + sens_num ].isConnected;
                break;
            case 5:
                MB_DATA->Coils.status_tSens[i].state_val_bit.Temp6_isConnected = sens[i * 16 + sens_num ].isConnected;
                break;
            case 6:
                MB_DATA->Coils.status_tSens[i].state_val_bit.Temp7_isConnected = sens[i * 16 + sens_num ].isConnected;
                break;
            case 7:
                MB_DATA->Coils.status_tSens[i].state_val_bit.Temp8_isConnected = sens[i * 16 + sens_num ].isConnected;
                break;
            case 8:
                MB_DATA->Coils.status_tSens[i].state_val_bit.Temp9_isConnected = sens[i * 16 + sens_num ].isConnected;
                break;
            case 9:
                MB_DATA->Coils.status_tSens[i].state_val_bit.Temp10_isConnected = sens[i * 16 + sens_num ].isConnected;
                break;
            case 10:
                MB_DATA->Coils.status_tSens[i].state_val_bit.Temp11_isConnected = sens[i * 16 + sens_num ].isConnected;
                break;
            case 11:
                MB_DATA->Coils.status_tSens[i].state_val_bit.Temp12_isConnected = sens[i * 16 + sens_num ].isConnected;
                break;
            case 12:
                MB_DATA->Coils.status_tSens[i].state_val_bit.Temp13_isConnected = sens[i * 16 + sens_num ].isConnected;
                break;
            case 13:
                MB_DATA->Coils.status_tSens[i].state_val_bit.Temp14_isConnected = sens[i * 16 + sens_num ].isConnected;
                break;
            case 14:
                MB_DATA->Coils.status_tSens[i].state_val_bit.Temp15_isConnected = sens[i * 16 + sens_num ].isConnected;
                break;
            case 15:
                MB_DATA->Coils.status_tSens[i].state_val_bit.Temp16_isConnected = sens[i * 16 + sens_num ].isConnected;
                break;
            }
        }
    }
    return FuncOK;

}

static uint16_t clamp_room_percent(uint16_t value)
{
    return value > 100U ? 100U : value;
}

static void update_room_modbus(MB_DataStructureTypeDef* MB_DATA)
{
    for (int i = 0; i < MAX_SENSE; i++)
    {
        MB_RoomInputRegsTypeDef* room = &MB_DATA->InRegs.room[i];
        MB_RoomHoldingRegsTypeDef* cfg = &MB_DATA->HoldRegs.room_cfg[i];
        uint16_t open_state = 0U;
        uint16_t close_state = 0U;
        uint16_t position_pct;
        uint16_t angle_max;

        if (cfg->valve_angle_max_deg == 0U)
        {
            cfg->valve_angle_max_deg = MB_ROOM_VALVE_ANGLE_MAX_DEFAULT;
        }

        if (i < 16)
        {
            open_state = (MB_DATA->Coils.relay_struct_on.all >> i) & 0x1U;
            close_state = (MB_DATA->Coils.relay_struct_off.all >> i) & 0x1U;
        }

        position_pct = clamp_room_percent(cfg->valve_position_pct);
        if (cfg->mode == ROOM_MODE_AUTO)
        {
            if (open_state)
            {
                position_pct = 100U;
            }
            else if (close_state)
            {
                position_pct = 0U;
            }
        }
        cfg->valve_position_pct = position_pct;
        angle_max = cfg->valve_angle_max_deg;

        room->channel = (uint16_t)(i + 1);
        room->location = cfg->location;
        for (int reg = 0; reg < 4; reg++)
        {
            room->ds18b20_id[reg] = ((uint16_t)MB_DATA->InRegs.ID.DevAddr[i][reg * 2]) |
                                    ((uint16_t)MB_DATA->InRegs.ID.DevAddr[i][reg * 2 + 1] << 8);
        }
        room->temperature_x10 = (i < hdallas.onewire->RomCnt) ? (uint16_t)((int16_t)(sens[i].temperature * MB_ROOM_TEMP_SCALE)) : 0U;
        room->setpoint_x10 = MB_DATA->HoldRegs.set_Temp[i];
        room->hysteresis_x10 = MB_DATA->HoldRegs.set_hyst[i];
        room->valve_position_pct = position_pct;
        room->valve_angle_deg = (uint16_t)((position_pct * angle_max) / 100U);
        room->valve_angle_max_deg = angle_max;
        room->is_connected = (i < hdallas.onewire->RomCnt) ? (uint16_t)sens[i].isConnected : 0U;
        room->valve_open = open_state;
        room->valve_close = close_state;
        room->mode = cfg->mode;
        room->command_state = cfg->command;
        room->reserved = 0U;
    }
}
FuncStat Field_modbus(MB_DataStructureTypeDef* MB_DATA, Flags_TypeDef* flag)
{

    MB_DATA->InRegs.ID = *hdallas.ds_devices;
    RTC_ModbusService(MB_DATA);
    flag->init_tsens = MB_DATA->Coils.init_Tsens;
    packStruct(MB_DATA, MAX_SENSE / 16);
    if (_debug_init || MB_DATA->Coils.init_param)
    {
        _debug_init = 0;

        MB_DATA->Coils.init_param = 0;
        for(int i = 0; i < hdallas.onewire->RomCnt; i++)
        {            if (MB_DATA->HoldRegs.room_cfg[i].apply)
            {
                MB_DATA->HoldRegs.set_Temp[i] = MB_DATA->HoldRegs.room_cfg[i].setpoint_x10;
                MB_DATA->HoldRegs.set_hyst[i] = MB_DATA->HoldRegs.room_cfg[i].hysteresis_x10;
                MB_DATA->HoldRegs.room_cfg[i].apply = 0U;
            }
            MB_DATA->HoldRegs.room_cfg[i].setpoint_x10 = MB_DATA->HoldRegs.set_Temp[i];
            MB_DATA->HoldRegs.room_cfg[i].hysteresis_x10 = MB_DATA->HoldRegs.set_hyst[i];
            sens[i].set_temp = ((float)MB_DATA->HoldRegs.set_Temp[i]) / MB_ROOM_TEMP_SCALE;
            sens[i].hyst = ((float)MB_DATA->HoldRegs.set_hyst[i]) / MB_ROOM_TEMP_SCALE;
        }



    }

    update_room_modbus(MB_DATA);
    return FuncOK;
};


FuncStat value_control(void )
{

    for(int i = 0; i < hdallas.onewire->RomCnt; i++)
    {
        if (i < 16 && MB_DATA.HoldRegs.room_cfg[i].mode == ROOM_MODE_MANUAL)
        {
            uint16_t manual_pct = clamp_room_percent(MB_DATA.HoldRegs.room_cfg[i].valve_position_pct);
            if (MB_DATA.HoldRegs.room_cfg[i].command == ROOM_COMMAND_CLOSE)
            {
                manual_pct = 0U;
            }
            else if (MB_DATA.HoldRegs.room_cfg[i].command == ROOM_COMMAND_OPEN && manual_pct == 0U)
            {
                manual_pct = 100U;
            }
            MB_DATA.HoldRegs.room_cfg[i].valve_position_pct = manual_pct;
            if (manual_pct > 0U)
            {
                MB_DATA.Coils.relay_struct_off.all &= ~(1 << i);
                MB_DATA.Coils.relay_struct_on.all |= 1 << i;
            }
            else
            {
                MB_DATA.Coils.relay_struct_on.all &= ~(1 << i);
                MB_DATA.Coils.relay_struct_off.all |= 1 << i;
            }
            continue;
        }

        if (sens[i].temperature < sens[i].set_temp - sens[i].hyst)

        {

            MB_DATA.Coils.relay_struct_off.all |= 1 << i;
            MB_DATA.Coils.relay_struct_on.all &= ~(1 << i);


        }

        else

            if (sens[i].temperature > sens[i].set_temp + sens[i].hyst)
            {

                MB_DATA.Coils.relay_struct_off.all &= ~(1 << i);
                MB_DATA.Coils.relay_struct_on.all |= 1 << i;

            }
						
				else

            if (sens[i].temperature == sens[i].set_temp )
						{
							MB_DATA.Coils.relay_struct_on.all &= ~(1 << i);
							MB_DATA.Coils.relay_struct_off.all &= ~(1 << i);
							
						}


    }


    return  FuncOK;
}


uint16_t handle_valves(TEMP_TypeDef* temp_sense[MAX_SENSE] )
{

    if (temp_sense[0]->state == STATE_OPEN_VALVE)
    {
        GPIOC->ODR |= 1 << 14;
    }
    else if (temp_sense[0]->state == STATE_CLOSE_VALVE)
    {
        GPIOC->ODR &= ~(1 << 14);
    }

    return 1;

}

void init_setpoint_all_T_sense(TEMP_TypeDef* temp_sense, int size_array)
{
    //ds_search_devices();
    for(int i = 0; i < size_array ; i++)
    {
        temp_sense[i].id[0] = roms[i][0] << 0 | roms[i][1] << 8 | roms[i][2] << 16 | roms[i][3] << 24;
        temp_sense[i].id[1] = roms[i][4] << 0 | roms[i][5] << 8 | roms[i][6] << 16 | roms[i][7] << 24;
        temp_sense[i].count = i + 1;
        temp_sense[i].location = 1;
        temp_sense[i].t_open = 22;
        temp_sense[i].t_close = 18;
        temp_sense[i].status_T_sense = 1;
    }
}
/* USER CODE END 4 */

/**
  * @brief  Period elapsed callback in non blocking mode
  * @note   This function is called  when TIM3 interrupt took place, inside
  * HAL_TIM_IRQHandler(). It makes a direct call to HAL_IncTick() to increment
  * a global variable "uwTick" used as application time base.
  * @param  htim : TIM handle
  * @retval None
  */
void HAL_TIM_PeriodElapsedCallback(TIM_HandleTypeDef *htim)
{
  /* USER CODE BEGIN Callback 0 */

  /* USER CODE END Callback 0 */
  if (htim->Instance == TIM3)
  {
    HAL_IncTick();
  }
  /* USER CODE BEGIN Callback 1 */

  /* USER CODE END Callback 1 */
}

/**
  * @brief  This function is executed in case of error occurrence.
  * @retval None
  */
void Error_Handler(void)
{
  /* USER CODE BEGIN Error_Handler_Debug */
    /* User can add his own implementation to report the HAL error return state */
    __disable_irq();
    while (1)
    {
    }
  /* USER CODE END Error_Handler_Debug */
}
#ifdef USE_FULL_ASSERT
/**
  * @brief  Reports the name of the source file and the source line number
  *         where the assert_param error has occurred.
  * @param  file: pointer to the source file name
  * @param  line: assert_param error line source number
  * @retval None
  */
void assert_failed(uint8_t *file, uint32_t line)
{
  /* USER CODE BEGIN 6 */
    /* User can add his own implementation to report the file name and line number,
       ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */
  /* USER CODE END 6 */
}
#endif /* USE_FULL_ASSERT */
